#include "application.h"

#include <QDir>
#include <stdexcept>
#include "settings/settingsdialog.h"

XkorApplication::XkorApplication(int & c, char * * v) : QApplication(c, v)
{
	configModified = eventModified = startListModified = tableGeneratorModified = false;
	
	settings = new QSettings("thirdgeek.com", "xkoranate");
	
	setDefaultSettings(); // in case we haven’t run the application before
	refreshSearchPaths(); // set the search path for sport files
	
	mainWindow = new QMainWindow;
	
	// create the widgets for each page
	ee = new XkorEventEditor;
	ce = new XkorConfigEditor;
	sle = new XkorStartListEditor;
	tg = new XkorTableGenerator;
	
	// assemble the tab view
	tw = new QTabWidget;
	tw->setUsesScrollButtons(false);
	tw->addTab(ee, "Scorinate Event");
	tw->addTab(ce, "Edit Config");
	tw->addTab(sle, "Edit Start List");
	tw->addTab(tg, "Generate Tables");
	
	// set up actions
	openAction = new QAction(QIcon(":/icons/document-open"), XkorApplication::tr("Open file…"), this);
	openAction->setShortcut(QKeySequence::Open);
	connect(openAction, SIGNAL(triggered()), this, SLOT(openFile()));
	
	saveAction = new QAction(QIcon(":/icons/document-save"), XkorApplication::tr("Save file…"), this);
	saveAction->setShortcut(QKeySequence::Save);
	connect(saveAction, SIGNAL(triggered()), this, SLOT(saveFile()));
	
	saveAsAction = new QAction(QIcon(":/icons/document-save-as"), XkorApplication::tr("Save file as…"), this);
	saveAsAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_S);
	connect(saveAsAction, SIGNAL(triggered()), this, SLOT(saveFileAs()));
	
	settingsAction = new QAction(QIcon(":/icons/configure"), tr("Preferences"), this);
	connect(settingsAction, SIGNAL(triggered()), this, SLOT(editSettings()));
	
	// toolbar
	toolBar = new QToolBar;
	toolBar->setMovable(false);
	toolBar->addAction(openAction);
	toolBar->addAction(saveAction);
	toolBar->addAction(saveAsAction);
	toolBar->addAction(settingsAction);
	
	mainWindow->setCentralWidget(tw);
	mainWindow->addToolBar(toolBar);
	updateTitle();
	
	connect(tw, SIGNAL(currentChanged(int)), this, SLOT(updateButtons()));
	connect(tw, SIGNAL(currentChanged(int)), this, SLOT(updateModifiedStatus()));
	connect(tw, SIGNAL(currentChanged(int)), this, SLOT(updateTitle()));
	connect(ee, SIGNAL(fileChanged(QString)), this, SLOT(setEventFile(QString)));
	connect(ee, SIGNAL(fileEdited(bool)), this, SLOT(setEventModified(bool)));
	connect(ce, SIGNAL(fileChanged(QString)), this, SLOT(setConfigFile(QString)));
	connect(ce, SIGNAL(fileEdited(bool)), this, SLOT(setConfigModified(bool)));
	connect(sle, SIGNAL(fileChanged(QString)), this, SLOT(setStartListFile(QString)));
	connect(sle, SIGNAL(fileEdited(bool)), this, SLOT(setStartListModified(bool)));
	connect(tg, SIGNAL(fileChanged(QString)), this, SLOT(setTableGeneratorFile(QString)));
	connect(tg, SIGNAL(fileEdited(bool)), this, SLOT(setTableGeneratorModified(bool)));
	
	mainWindow->setWindowIcon(QIcon(":/icons/xkoranate"));
	
	mainWindow->show();
}

void XkorApplication::editSettings()
{
	XkorSettingsDialog sd(mainWindow);
	sd.setSportDirectory(settings->value("sportDirectory").toString());
	int rval = sd.exec();
	if(rval == QDialog::Accepted)
	{
		settings->setValue("sportDirectory", sd.getSportDirectory());
		refreshSearchPaths();
		ee->updateSports(); // update the list of sports in the event editor to reflect the potentially-new sport directory
	}
}

bool XkorApplication::notify(QObject * rec, QEvent * ev)
{
	try
	{
		return QApplication::notify(rec, ev);
	}
	catch(std::exception & e)
	{
		qFatal(e.what());
		return false;
	}
	catch(...)
	{
		qFatal("Unknown exception!");
		abort();
	}
}

void XkorApplication::openFile()
{
	QWidget * widget = tw->currentWidget();
	if(widget == ee)
		ee->openFile();
	else if(widget == ce)
		ce->openFile();
	else if(widget == sle)
		sle->openFile();
	else if(widget == tg)
		tg->openFile();
}

void XkorApplication::refreshSearchPaths()
{
	QDir::setSearchPaths("sports", QStringList(settings->value("sportDirectory").toString()));
}

void XkorApplication::saveFile()
{
	QWidget * widget = tw->currentWidget();
	if(widget == ee)
		ee->saveFile();
	else if(widget == ce)
		ce->saveFile();
	else if(widget == sle)
		sle->saveFile();
	else if(widget == tg)
		tg->saveFile();
}

void XkorApplication::saveFileAs()
{
	QWidget * widget = tw->currentWidget();
	if(widget == ee)
		ee->saveFileAs();
	else if(widget == ce)
		ce->saveFileAs();
	else if(widget == sle)
		sle->saveFileAs();
	else if(widget == tg)
		tg->saveFileAs();
}

void XkorApplication::setConfigFile(QString filename)
{
	configFile = filename;
	updateTitle();
}

void XkorApplication::setConfigModified(bool isModified)
{
	configModified = isModified;
	updateModifiedStatus();
}

void XkorApplication::setDefaultSettings()
{
	if(!settings->contains("sportDirectory"))
		settings->setValue("sportDirectory", QDir::currentPath() + "/sports/");
}

void XkorApplication::setEventFile(QString filename)
{
	eventFile = filename;
	updateTitle();
}

void XkorApplication::setEventModified(bool isModified)
{
	eventModified = isModified;
	updateModifiedStatus();
}

void XkorApplication::setStartListFile(QString filename)
{
	startListFile = filename;
	updateTitle();
}

void XkorApplication::setStartListModified(bool isModified)
{
	startListModified = isModified;
	updateModifiedStatus();
}

void XkorApplication::setTableGeneratorFile(QString filename)
{
	tableGeneratorFile = filename;
	updateTitle();
}

void XkorApplication::setTableGeneratorModified(bool isModified)
{
	tableGeneratorModified = isModified;
	updateModifiedStatus();
}

void XkorApplication::updateButtons()
{
	QWidget * widget = tw->currentWidget();
	bool isEE = (widget == ee);
	bool isCE = (widget == ce);
	bool isSLE = (widget == sle);
	bool isTG = (widget == tg);
	
	openAction->setEnabled(isEE || isCE || isSLE || isTG);
	saveAction->setEnabled(isEE || isCE || isSLE || isTG);
	saveAsAction->setEnabled(isEE || isCE || isSLE || isTG);
}

void XkorApplication::updateModifiedStatus()
{
	QWidget * widget = tw->currentWidget();
	if(widget == ee)
		mainWindow->setWindowModified(eventModified);
	else if(widget == ce)
		mainWindow->setWindowModified(configModified);
	else if(widget == sle)
		mainWindow->setWindowModified(startListModified);
	else if(widget == tg)
		mainWindow->setWindowModified(tableGeneratorModified);
}

void XkorApplication::updateTitle()
{
	QWidget * widget = tw->currentWidget();
	QString fileName = tr("Untitled");
	if(widget == ee)
	{
		if(eventFile != "")
			fileName = QFileInfo(eventFile).fileName();
		mainWindow->setWindowTitle(fileName + tr("[*] – xkoranate"));
	}
	else if(widget == ce)
	{
		if(configFile != "")
			fileName = QFileInfo(configFile).fileName();
		mainWindow->setWindowTitle(fileName + tr("[*] – xkoranate"));
	}
	else if(widget == sle)
	{
		if(startListFile != "")
			fileName = QFileInfo(startListFile).fileName();
		mainWindow->setWindowTitle(fileName + tr("[*] – xkoranate"));
	}
	else if(widget == tg)
	{
		if(tableGeneratorFile != "")
			fileName = QFileInfo(tableGeneratorFile).fileName();
		mainWindow->setWindowTitle(fileName + tr("[*] – xkoranate"));
	}
}
