#include "eventeditor.h"

#include <QFile>
#include <QFileDialog>
#include <QFormLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QMessageBox>
#include <QTextCodec>
#include <QTextStream>
#include <QtDebug>
#include "event.h"
#include "rpfile.h"
#include "xml_index.h"

XkorEventEditor::XkorEventEditor(QWidget * parent) : QWidget(parent)
{
	fileIsModified = false;
	
	outputFormatNames << tr("Default") << tr("Alpine skiing") << tr("Biathlon") << tr("Bobsleigh") << tr("Cross-country skiing") << tr("Curling") << tr("Figure skating") << tr("Freestyle skiing (aerials, moguls)") << tr("Freestyle skiing (ski cross)") << tr("Ice hockey") << tr("Ice hockey (with overtime/shootout)") << tr("Luge") << tr("Short track speed skating") << tr("Skeleton") << tr("Ski jumping") << tr("Snowboard (halfpipe)") << tr("Snowboard (parallel giant slalom qual.)") << tr("Snowboard (parallel giant slalom KO)") << tr("Snowboard (snowboard cross)") << tr("Speed skating") << tr("Speed skating (team pursuit KO)");
	outputFormatCodes << "standard" << "alpineSkiing" << "biathlon" << "bobsleigh" << "crossCountrySkiing" << "curling" << "figureSkating" << "freestyleSkiing" << "skiCross" << "iceHockey" << "iceHockey-ot" << "luge" << "shortTrack" << "skeleton" << "skiJumping" << "halfpipe" << "pgsQual" << "pgs" << "snowboardCross" << "speedSkating" << "speedSkatingTeamPursuit";
	
	eventName = new QLineEdit;
	eventConfig = new XkorFileLineEdit(tr("Select configuration file"), ".", tr("XML files (*.xml)"));
	sport = new QComboBox;
	outputFormat = new QComboBox;
	outputFormat->addItems(outputFormatNames);
	
	updateSports();
	
	refreshAction = new QAction(QIcon(":/icons/view-refresh"), tr("Refresh configuration"), this);
	refreshAction->setShortcut(QKeySequence::Refresh);
	connect(refreshAction, SIGNAL(triggered()), this, SLOT(updateAvailableStartLists()));
	
	// if the config file is changed, we need to refresh
	connect(eventConfig, SIGNAL(editingFinished()), refreshAction, SLOT(trigger()));
	
	// give the user a button to refresh
	refreshConfig = new QToolButton;
	refreshConfig->setDefaultAction(refreshAction);
	
	// lay out the refresh button
	QHBoxLayout * configLayout = new QHBoxLayout;
	configLayout->addWidget(eventConfig);
	configLayout->addWidget(refreshConfig);
	configLayout->setContentsMargins(0, 0, 0, 0);
	
	slw = new XkorStartListWidget;
	connect(slw, SIGNAL(listChanged()), this, SLOT(updateAthletes()));
	
	aw = new XkorEventSetupWidget;
	
	// form layout
	QFormLayout * form = new QFormLayout;
	form->addRow(tr("Name:"), eventName);
	form->addRow(tr("Config file:"), configLayout);
	form->addRow(tr("Sport name:"), sport);
	form->addRow(tr("Output format:"), outputFormat);
	
	// start lists tree widget layout
	QGroupBox * slgroup = new QGroupBox(tr("Start lists"));
	QHBoxLayout * sllayout = new QHBoxLayout;
	sllayout->addWidget(slw);
	slgroup->setLayout(sllayout);
	
	// athletes tree widget layout
	QGroupBox * agroup = new QGroupBox(tr("Athletes"));
	QHBoxLayout * alayout = new QHBoxLayout;
	alayout->addWidget(aw);
	agroup->setLayout(alayout);
	
	// scorinator
	scorw = new XkorScorinateWidget;
	
	scorinateAction = new QAction(QIcon(":/icons/roll"), tr("Scorinate event"), this);
	connect(scorinateAction, SIGNAL(triggered()), this, SLOT(scorinateEvent()));
	
	exportResultsAction = new QAction(QIcon(":/icons/document-export"), tr("Export to file"), this);
	connect(exportResultsAction, SIGNAL(triggered()), this, SLOT(exportResults()));
	
	scorinate = new QToolButton;
	scorinate->setDefaultAction(scorinateAction);
	
	exportResultsButton = new QToolButton;
	exportResultsButton->setDefaultAction(exportResultsAction);
	
	QGroupBox * scorgroup = new QGroupBox(tr("Scorinate"));
	QGridLayout * scorlayout = new QGridLayout;
	scorlayout->addWidget(scorw, 0, 0, 5, 1);
	scorlayout->addWidget(scorinate, 1, 1);
	scorlayout->addWidget(exportResultsButton, 3, 1);
	scorgroup->setLayout(scorlayout);
	
	// layout
	QGridLayout * layout = new QGridLayout(this);
	layout->addLayout(form, 0, 0);
	layout->addWidget(slgroup, 0, 1);
	layout->addWidget(agroup, 1, 0);
	layout->addWidget(scorgroup, 1, 1);
	
	layout->setRowMinimumHeight(0, form->sizeHint().height());
	
	layout->setRowStretch(0, 0);
	layout->setRowStretch(1, 1093);
	
	connect(eventName, SIGNAL(textChanged(QString)), this, SLOT(setModified()));
	connect(eventConfig, SIGNAL(textChanged(QString)), this, SLOT(setModified()));
	connect(sport, SIGNAL(currentIndexChanged(int)), this, SLOT(setModified()));
	connect(outputFormat, SIGNAL(textChanged(QString)), this, SLOT(setModified()));
	connect(slw, SIGNAL(listChanged()), this, SLOT(setModified()));
	connect(aw, SIGNAL(listChanged()), this, SLOT(setModified()));
}

void XkorEventEditor::disableSports(QString reason)
{
	sport->clear();
	sport->insertItem(0, reason);
	sport->setEnabled(false);
}

void XkorEventEditor::exportResults()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save results"), ".", tr("Text files (*.txt)"));
	if(filename != QString())
	{
		QFile f(filename);
		scorw->writeResults(&f);
	}
}

QBuffer * XkorEventEditor::getFile()
{
	QBuffer * rval = new QBuffer;
	writeFile(rval);
	return rval;
}

bool XkorEventEditor::okayToLoad()
{
	if(fileIsModified)
	{
		int r = QMessageBox::warning(this, tr("xkoranate"), tr("The event file has been modified.\nDo you want to save your changes?"), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel | QMessageBox::Escape);
		if(r == QMessageBox::Cancel)
			return false;
	}
	return true; // if the current file hasn’t been modified, or if the user clicked Yes or No
}

void XkorEventEditor::openFile()
{
	if(okayToLoad())
	{
		QString filename = QFileDialog::getOpenFileName(this, tr("Open event file"), ".", tr("XML files (*.xml)"));
		
		try
		{
			XkorEvent e(filename);
			
			slw->clear();
			aw->clear();
			eventName->setText(e.getName());
			eventConfig->setText(e.getConfigFileName());
			updateSports();
			updateAvailableStartLists();
			sport->setCurrentIndex(sport->findText(e.getSport()));
			outputFormat->setCurrentIndex(outputFormatCodes.indexOf(e.getOutputType()));
			
			// start lists
			slw->setStartLists(e.getStartLists());
			
			// athletes
			aw->setGroups(e.getGroups());
			
			currentFileName = filename;
			emit fileChanged(filename);
			setModified(false);
		}
		catch(XkorFileNotFoundException & e)
		{
			qDebug() << e.what();
			if(e.getFileName() != QString())
				QMessageBox::critical(this, tr("xkoranate"), tr("Specified %1 (%2) not found.").arg(e.getFileType()).arg(e.getFileName()));
		}
	}
}

void XkorEventEditor::saveFile(QString filename)
{	
	if(filename == QString()) // if no filename was specified
	{
		if(currentFileName == QString()) // still “Untitled”?
			saveFileAs();
		else
			filename = currentFileName;
	}
	QFile f(filename);
	writeFile(&f);
	currentFileName = filename;
	emit fileChanged(filename);
	setModified(false);
}

void XkorEventEditor::saveFileAs()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save event file"), ".", tr("XML files (*.xml)"));
	if(filename != QString())
		saveFile(filename);
}

void XkorEventEditor::scorinateEvent()
{
	QBuffer * buf = new QBuffer;
	writeFile(buf);
	scorw->scorinate(buf);
	delete buf;
}

void XkorEventEditor::setModified(bool isModified)
{
	fileIsModified = isModified;
	emit fileEdited(isModified);
}

void XkorEventEditor::updateAthletes()
{
	std::vector<XkorStartList> slvec;
	try
	{
		XkorConfig conf(eventConfig->text());
		XkorXmlIndex startlist_ind;
		startlist_ind.traverse(conf.getStartListDirectory());
		slvec.clear();
		std::vector<QString> startLists = slw->startLists();
		for(std::vector<QString>::iterator i = startLists.begin(); i != startLists.end(); ++i)
		{
			try
			{
				// look up each start list in the tree widget and store it
				slvec.push_back(XkorStartList(startlist_ind.lookup(*i)));
			}
			catch(XkorSearchFailedException & e) {}
		}
	}
	catch(XkorFileNotFoundException & e)
	{
		qDebug() << e.what();
		slvec.clear();
	}
	aw->setStartLists(slvec);
}

void XkorEventEditor::updateAvailableStartLists()
{
	try
	{
		QStringList availableStartLists;
		XkorConfig conf(eventConfig->text());
		XkorXmlIndex startlist_ind;
		startlist_ind.traverse(conf.getStartListDirectory());
		std::map<QString, QString> l = startlist_ind.getAllFiles();
		for(std::map<QString, QString>::iterator i = l.begin(); i != l.end(); ++i)
			availableStartLists << i->first;
		slw->setAvailableStartLists(availableStartLists);
	}
	catch(XkorFileNotFoundException & e)
	{
		qDebug() << e.what();
		slw->setAvailableStartLists(QStringList());
	}
}

void XkorEventEditor::updateSports()
{
	// load the config file
	try
	{
		QString currentSport = sport->currentText();
		
		// load the sports
		XkorXmlIndex x;
		x.traverse("sports:./");
		
		std::map<QString, QString> s = x.getAllFiles();
		sport->clear();
		QStringList l;
		for(std::map<QString, QString>::iterator i = s.begin(); i != s.end(); ++i)
			l << i->first;
		
		// insert the items in the combobox
		sport->insertItems(0, l);
		
		// keep the previous selection if it’s still there
		int index = sport->findText(currentSport);
		if(index >= 0)
			sport->setCurrentIndex(index);
	}
	catch(XkorFileNotFoundException e)
	{
		disableSports(tr("Specified %1 does not exist").arg(e.getFileType()));
	}
}

void XkorEventEditor::writeFile(QIODevice * dev)
{
	dev->open(QIODevice::WriteOnly);
	
	QTextStream ost(dev);
	ost.setCodec(QTextCodec::codecForName("UTF-8"));
	
	ost << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<event>\n";
	ost << "\t<name>" << eventName->text() << "</name>\n";
	ost << "\t<rpbonus>" << eventConfig->text() << "</rpbonus>\n";
	ost << "\t<sport>" << sport->currentText() << "</sport>\n";
	
	// find the code for the output format
	QString outputCode;
	int n = outputFormat->currentIndex();
	if(n >= 0 && n < outputFormatCodes.size())
		outputCode = outputFormatCodes[n];
	ost << "\t<output>" << outputCode << "</output>\n";
	
	ost << "\t<startLists>\n";
	std::vector<QString> startLists = slw->startLists();
	for(std::vector<QString>::iterator i = startLists.begin(); i != startLists.end(); ++i)
	{
		ost << "\t\t<list>" << *i << "</list>\n";
	}
	ost << "\t</startLists>\n";
	std::vector<XkorGroup> groups = aw->groups();
	for(std::vector<XkorGroup>::iterator i = groups.begin(); i != groups.end(); ++i)
	{
		ost << "\t<group>\n";
		ost << "\t\t<groupName>" << i->name << "</groupName>\n";
		for(std::vector<QString>::iterator j = i->athletes.begin(); j != i->athletes.end(); ++j)
			ost << "\t\t<athlete>" << *j << "</athlete>\n";
		ost << "\t</group>\n";
	}
	ost << "</event>\n";
}
