#include "tablegenerator/table.h"


#include <algorithm>
#include <QFile>
#include <QTextCodec>
#include <QTextStream>
#include "xml/xmltablehandler.h"

XkorTable::XkorTable(QString & filename) throw(XkorFileNotFoundException)
{
	data.push_back(std::vector<XkorTableRow>());
	
	QFile * f = new QFile(filename);
	if(!f->exists())
	{
		std::string err = "File ‘";
		err += filename.toStdString();
		err += "’ not found by XkorTable::XkorTable(const QString &)";
		throw XkorFileNotFoundException(err, QObject::tr("table file"), filename);
	}
	QXmlInputSource * in = new QXmlInputSource(f);
	
	// parse the XML file
	QXmlSimpleReader r;
	XkorXmlTableHandler * h = new XkorXmlTableHandler;
	r.setContentHandler(h);
	r.parse(in, false);
	
	tableName = h->getName();
	columns = h->getColumns();
	matches = h->getMatches();
	pointsForWin = h->getPointsForWin();
	pointsForDraw = h->getPointsForDraw();
	pointsForLoss = h->getPointsForLoss();
	sortCriteria = h->getSortCriteria();
	
	sorter.setPointsForWin(pointsForWin);
	sorter.setPointsForDraw(pointsForDraw);
	sorter.setPointsForLoss(pointsForLoss);
	
	updateMatches();
	
	delete h;
	delete in;
	delete f;
}

void XkorTable::collapse(std::vector<std::vector<XkorTableRow> > & target, const std::vector<std::vector<XkorTableRow> > & source)
{
	for(std::vector<std::vector<XkorTableRow> >::const_iterator i = source.begin(); i != source.end(); ++i)
		target.push_back(*i);
}

std::vector<XkorTableRow>::iterator XkorTable::findTeam(QString & name, int index)
{
	for(std::vector<XkorTableRow>::iterator i = data[index].begin(); i != data[index].end(); ++i)
	{
		if(i->name() == name)
			return i;
	}
	// if there is no team with that name
	data[index].push_back(XkorTableRow(name));
	return --(data[index].end());
}

void XkorTable::generate()
{
	if(data.size() > 0)
	{
		// collapse the existing table, as it might already be sorted
		std::vector<std::vector<XkorTableRow> > oldData;
		oldData.push_back(std::vector<XkorTableRow>()); // just one row
		for(std::vector<std::vector<XkorTableRow> >::iterator i = data.begin(); i != data.end(); ++i)
			for(std::vector<XkorTableRow>::iterator j = i->begin(); j != i->end(); ++j)
				oldData[0].push_back(*j);
		
		std::vector<std::vector<XkorTableRow> > newData;
		for(std::vector<std::vector<XkorTableRow> >::iterator i = oldData.begin(); i != oldData.end(); ++i)
		{
			collapse(newData, sortTable(*i));
		}
		data = newData;
	}
}

double XkorTable::getPoints(const XkorTableRow & a)
{
	return a.wins() * pointsForWin + a.draws() * pointsForDraw + a.losses() * pointsForLoss;
}

std::vector<std::vector<XkorTableRow> > XkorTable::sortTable(const std::vector<XkorTableRow> & rows)
{
	std::vector<std::vector<XkorTableRow> > rval;
	
	if(rows.size() > 0)
	{
		std::vector<std::vector<XkorTableRow> > sorted;
		
		for(std::vector<QString>::iterator i = sortCriteria.begin(); i != sortCriteria.end(); ++i)
		{
			sorted = sorter.sort(rows, *i);
			if(sorted.size() > 1)
			{
				for(std::vector<std::vector<XkorTableRow> >::iterator i = sorted.begin(); i != sorted.end(); ++i)
					collapse(rval, sortTable(*i));
				return rval;
			}
		}
		
		// if the teams are tied
		sorted = sorter.sort(rows, "alphabetical");
		rval.push_back(sorted[0]);
	}
	else
		rval.push_back(std::vector<XkorTableRow>());
	return rval;
}

QString XkorTable::toText()
{
	QString rval;
	
	// header
	for(std::vector<XkorTableColumn>::iterator k = columns.begin(); k != columns.end(); ++k)
	{
		if(k->columnType == "name")
			rval += (k->heading).leftJustified(k->width);
		else
			rval += (k->heading).rightJustified(k->width);
		rval += " ";
	}
	rval += "\n";
	
	int rank = 1;
	int acc = 1;
	for(std::vector<std::vector<XkorTableRow> >::iterator i = data.begin(); i != data.end(); ++i)
	{
		for(std::vector<XkorTableRow>::iterator j = i->begin(); j != i->end(); ++j)
		{
			for(std::vector<XkorTableColumn>::iterator k = columns.begin(); k != columns.end(); ++k)
			{
				if(k != columns.begin())
					rval += " ";
				if(k->columnType == "position")
				{
					if(rank == acc)
						rval += QString::number(rank).rightJustified(k->width);
					else
						rval += QString(k->width, ' ');
				}
				else if(k->columnType == "name")
					rval += j->name().leftJustified(k->width);
				else if(k->columnType == "played")
					rval += QString::number(j->played()).rightJustified(k->width);
				else if(k->columnType == "wins")
					rval += QString::number(j->wins()).rightJustified(k->width);
				else if(k->columnType == "draws")
					rval += QString::number(j->draws()).rightJustified(k->width);
				else if(k->columnType == "losses")
					rval += QString::number(j->losses()).rightJustified(k->width);
				else if(k->columnType == "goalsFor")
					rval += QString::number(j->goalsFor()).rightJustified(k->width);
				else if(k->columnType == "goalsAgainst")
					rval += QString::number(j->goalsAgainst()).rightJustified(k->width);
				else if(k->columnType == "goalDifference")
					rval += (QString(j->goalDifference() > 0 ? "+" : (j->goalDifference() < 0 ? "−" : "")) + QString::number(abs((int)j->goalDifference()))).rightJustified(k->width);
				else if(k->columnType == "goalAverage")
					rval += (j->goalsAgainst() == 0 ? QString("∞") : QString::number(j->goalsFor() / j->goalsAgainst(), 'f', 3)).rightJustified(k->width);
				else if(k->columnType == "points")
					rval += QString::number(getPoints(*j)).rightJustified(k->width);
				else if(k->columnType == "winPercent")
					rval += (j->played() == 0 ? QString("undef") : QString::number(j->wins() / j->played(), 'f', 3)).rightJustified(k->width);
				else if(k->columnType == "resultsGrid")
				{
					QString gridLine;
					for(std::vector<std::vector<XkorTableRow> >::iterator y = data.begin(); y != data.end(); ++y)
						for(std::vector<XkorTableRow>::iterator z = y->begin(); z != y->end(); ++z)
						{
							if(y != data.begin())
								gridLine += "  ";
							std::pair<double, double> result = j->result(z->name());
							if(result.first >= 0)
								gridLine += QString::number(result.first) + "–" + QString::number(result.second);
							else
								gridLine += " — ";
						}
					rval += gridLine.rightJustified(k->width);
				}
			}
			rval += "\n";
			++acc;
		}
		rank = acc;
	}
	return rval;
}

void XkorTable::updateMatches()
{
	data.clear();
	data.push_back(std::vector<XkorTableRow>());
	for(std::vector<XkorTableMatch>::iterator i = matches.begin(); i != matches.end(); ++i)
	{
		std::vector<XkorTableRow>::iterator j = findTeam(i->team1);
		j->insertMatch(i->team2, i->score1, i->score2, true);
		j = findTeam(i->team2);
		j->insertMatch(i->team1, i->score2, i->score1, false);
	}
}

void XkorTable::writeFile(QIODevice * dev)
{
	dev->open(QIODevice::WriteOnly);
	
	QTextStream ost(dev);
	ost.setCodec(QTextCodec::codecForName("UTF-8"));
	
	ost << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<event>\n";
	ost << "<table>\n";
	ost << "\t<name>" << tableName << "</name>\n";
	ost << "\t<pointsForWin>" << QString::number(pointsForWin) << "</pointsForWin>\n";
	ost << "\t<pointsForDraw>" << QString::number(pointsForDraw) << "</pointsForDraw>\n";
	ost << "\t<pointsForLoss>" << QString::number(pointsForLoss) << "</pointsForLoss>\n";
	ost << "\t<sortCriteria>\n";
	for(std::vector<QString>::iterator i = sortCriteria.begin(); i != sortCriteria.end(); ++i)
		ost << "\t\t<sortCriterion>" << *i << "</sortCriterion>\n";
	ost << "\t</sortCriteria>\n";
	for(std::vector<XkorTableColumn>::iterator i = columns.begin(); i != columns.end(); ++i)
		ost << "\t<column type=\"" << i->columnType << "\" width=\"" << QString::number(i->width) << "\">" << i->heading << "</column>\n";
	for(std::vector<XkorTableMatch>::iterator i = matches.begin(); i != matches.end(); ++i)
	{
		ost << "\t<match>\n";
		ost << "\t\t<home>" << i->team1 << "</home>\n";
		ost << "\t\t<homeScore>" << QString::number(i->score1) << "</homeScore>\n";
		ost << "\t\t<awayScore>" << QString::number(i->score2) << "</awayScore>\n";
		ost << "\t\t<away>" << i->team2 << "</away>\n";
		ost << "\t</match>\n";
	}
	ost << "</table>\n";
}
