#include "tablegenerator/tablegenerator.h"

#include <QFileDialog>
#include <QFormLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QMessageBox>
#include <QTextCodec>
#include <QTextStream>

XkorTableGenerator::XkorTableGenerator(QWidget * parent) : QWidget(parent)
{
	fileIsModified = false;
	
	pointsForWin = new QDoubleSpinBox;
	pointsForWin->setDecimals(0);
	pointsForDraw = new QDoubleSpinBox;
	pointsForDraw->setDecimals(0);
	pointsForLoss = new QDoubleSpinBox;
	pointsForLoss->setDecimals(0);
	
	mw = new XkorMatchWidget;
	scw = new XkorSortCriteriaWidget;
	tcw = new XkorTableColumnsWidget;
	
	QFormLayout * form = new QFormLayout;
	form->addRow(tr("Win:"), pointsForWin);
	form->addRow(tr("Draw:"), pointsForDraw);
	form->addRow(tr("Loss:"), pointsForLoss);
	
	QGroupBox * ptsGroup = new QGroupBox(tr("Point values"));
	ptsGroup->setLayout(form);
	
	QGroupBox * scGroup = new QGroupBox(tr("Sort criteria"));
	QHBoxLayout * scLayout = new QHBoxLayout;
	scLayout->addWidget(scw);
	scGroup->setLayout(scLayout);
	
	QGroupBox * tcGroup = new QGroupBox(tr("Column settings"));
	QHBoxLayout * tcLayout = new QHBoxLayout;
	tcLayout->addWidget(tcw);
	tcGroup->setLayout(tcLayout);
	
	QGroupBox * mGroup = new QGroupBox(tr("Matches"));
	QHBoxLayout * mLayout = new QHBoxLayout;
	mLayout->addWidget(mw);
	mGroup->setLayout(mLayout);
	
	textedit = new QTextEdit;
	doc = NULL;
	
	// layout
	QGridLayout * layout = new QGridLayout(this);
	layout->addWidget(scGroup, 0, 0, 1, 2);
	layout->addWidget(tcGroup, 0, 2);
	layout->addWidget(ptsGroup, 1, 0);
	layout->addWidget(textedit, 1, 1, 1, 2);
	layout->addWidget(mGroup, 2, 0, 1, 3);
	layout->setColumnStretch(0, 1);
	layout->setColumnStretch(1, 3);
	layout->setColumnStretch(2, 4);
	
	connect(pointsForWin, SIGNAL(valueChanged(QString)), this, SLOT(setPointsForWin(QString)));
	connect(pointsForDraw, SIGNAL(valueChanged(QString)), this, SLOT(setPointsForDraw(QString)));
	connect(pointsForLoss, SIGNAL(valueChanged(QString)), this, SLOT(setPointsForLoss(QString)));
	connect(mw, SIGNAL(listChanged()), this, SLOT(updateMatches()));
	connect(scw, SIGNAL(listChanged()), this, SLOT(updateSortCriteria()));
	connect(tcw, SIGNAL(listChanged()), this, SLOT(updateTableColumns()));
	connect(this, SIGNAL(fileEdited(bool)), this, SLOT(refreshTable(bool)));
}

XkorTableGenerator::~XkorTableGenerator()
{
	delete textedit;
	delete doc;
}

bool XkorTableGenerator::okayToLoad()
{
	if(fileIsModified)
	{
		int r = QMessageBox::warning(this, tr("xkoranate"), tr("The table file has been modified.\nDo you want to save your changes?"), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel | QMessageBox::Escape);
		if(r == QMessageBox::Cancel)
			return false;
	}
	return true; // if the current file hasn’t been modified, or if the user clicked Yes or No
}

void XkorTableGenerator::openFile()
{
	if(okayToLoad())
	{
		QString filename = QFileDialog::getOpenFileName(this, tr("Open table file"), ".", tr("XML files (*.xml)"));
		if(filename != "")
		{
			try
			{
				// load the event we’re running
				t = XkorTable(filename);
				refreshTable();
				
				pointsForWin->setValue(t.getPointsForWin());
				pointsForDraw->setValue(t.getPointsForDraw());
				pointsForLoss->setValue(t.getPointsForLoss());
				mw->setMatches(t.getMatches());
				scw->setSortCriteria(t.getSortCriteria());
				tcw->setColumns(t.getColumns());
				
				currentFileName = filename;
				emit fileChanged(filename);
				setModified(false);
			}
			catch(XkorFileNotFoundException & e)
			{
				qDebug() << e.what();
				QMessageBox::critical(this, tr("xkoranate"), tr("Specified %1 (%2) not found.").arg(e.getFileType()).arg(e.getFileName()));
			}
		}
	}
}

void XkorTableGenerator::refreshTable(bool isModified)
{
	if(isModified)
	{
		t.generate();
		
		// display
		textedit->setDocument(NULL);
		delete doc;
		
		doc = new QTextDocument;
		doc->setHtml("<pre>" + t.toText() + "</pre>");
		textedit->setDocument(doc);
	}
}

void XkorTableGenerator::saveFile(QString filename)
{	
	if(filename == QString()) // if no filename was specified
	{
		if(currentFileName == QString()) // still “Untitled”?
			saveFileAs();
		else
			filename = currentFileName;
	}
	QFile f(filename);
	writeFile(&f);
	currentFileName = filename;
	emit fileChanged(filename);
	setModified(false);
}

void XkorTableGenerator::saveFileAs()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save table file"), ".", tr("XML files (*.xml)"));
	if(filename != QString())
		saveFile(filename);
}

void XkorTableGenerator::setModified(bool isModified)
{
	fileIsModified = isModified;
	emit fileEdited(isModified);
}

void XkorTableGenerator::setPointsForWin(QString pts)
{
	t.setPointsForWin(pts.toDouble());
	setModified(true);
}

void XkorTableGenerator::setPointsForDraw(QString pts)
{
	t.setPointsForDraw(pts.toDouble());
	setModified(true);
}

void XkorTableGenerator::setPointsForLoss(QString pts)
{
	t.setPointsForLoss(pts.toDouble());
	setModified(true);
}

void XkorTableGenerator::updateMatches()
{
	t.setMatches(mw->matches());
	setModified(true);
}

void XkorTableGenerator::updateSortCriteria()
{
	t.setSortCriteria(scw->sortCriteria());
	setModified(true);
}

void XkorTableGenerator::updateTableColumns()
{
	t.setColumns(tcw->columns());
	setModified(true);
}

void XkorTableGenerator::writeFile(QIODevice * dev)
{
	t.writeFile(dev);
	setModified(true);
}
